package com.cweb.controller.meal;

import com.alibaba.fastjson.JSONObject;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.entity.BsMealTableOrder;
import com.hfkj.entity.BsMealTableOrderProduct;
import com.hfkj.model.ResponseData;
import com.hfkj.service.meal.BsMealShopCartService;
import com.hfkj.service.meal.BsMealTableOrderChildService;
import com.hfkj.service.meal.BsMealTableOrderProductService;
import com.hfkj.service.meal.BsMealTableOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @className: BsMealShopCartController
 * @author: HuRui
 * @date: 2023/11/30
 **/
@Controller
@Api(value = "桌台订单管理")
@RequestMapping(value = "/mealTableOrder")
public class BsMealTableOrderController {

    Logger log = LoggerFactory.getLogger(BsMealTableController.class);

    @Resource
    private BsMealTableOrderChildService mealTableOrderChildService;
    @Resource
    private BsMealTableOrderService mealTableOrderService;
    @Resource
    private BsMealTableOrderProductService mealTableOrderProductService;

    @RequestMapping(value="/createOrder",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "创建订单")
    public ResponseData createOrder(@RequestBody JSONObject body) {
        try {
            if (body == null || StringUtils.isBlank(body.getString("serialNumber"))) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            mealTableOrderChildService.createChildOrder(body.getString("serialNumber"), body.getString("userIdentity"));

            return ResponseMsgUtil.success("操作成功");

        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryCurrentOrder",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询当前订单")
    public ResponseData queryCurrentOrder(@RequestParam(value = "serialNumber" , required = true) String serialNumber) {
        try {
            // 查询桌台当前订单
            BsMealTableOrder tableOrder = mealTableOrderService.getCurrentTableOrder(serialNumber);
            if (tableOrder == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "当初桌台未开台");
            }
            Map<String, Object> map = new HashMap<>();
            map.put("order", tableOrder);

            Map<String,Object> param = new HashMap<>();
            param.put("orderNo", tableOrder.getOrderNo());
            map.put("product", mealTableOrderProductService.getOrderProductList(param));
            return ResponseMsgUtil.success(map);

        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return ResponseMsgUtil.exception(e);
        }
    }
}


